<?php
/**
 * @file
 * settings file
 */

/**
 * Implements hook_form().
 * admins acknowledgements for the current node. 
 */
function sign_for_acknowledgement_list_form()
{
  if (arg(0) == 'node' && is_numeric(arg(1))) {
    $nodeid = arg(1);
  }
  else {
    $nodeid = 0;
  }
  return _sign_for_acknowledgement_create_display_table($nodeid);
}
/**
 * @param timestamp $expire (the expiration date)
 * @param timestamp $signed (the signature date)
 * @return string describing the status of the signature.  
 */    
function _sign_for_acknowledgement_status($expire, $signed)
{
  $now = time();
  if ($signed) {
    if ($signed < $expire) {
      return t('signed ok...');
    }
    else {
      return t('signed out of terms...');
    }
  }
  else {
    if ($now < $expire) {
      return t('still to be signed...');
    }
    else {
      return t('terms have expired...');
    }
  }
}
/**
 * Users table & form creation.
 *
 * @param int $nodeid
 *   id of the current node
 *
 * @return array data (form)
 */
function _sign_for_acknowledgement_create_display_table($nodeid)
{
  if (!is_numeric($nodeid))
  {
    return array('#markup' => t('WARNING: incorrect node id...'));
  }
  $rows_limit = variable_get('sign_for_acknowledgement_limit', 10);

  // Get expiration datetime.
  $date = NULL;
  $timestamp = _sign_for_acknowledgement_get_expiration_date(TRUE);
  if ($timestamp) {
    $dateobject = new DateObject($timestamp);
    $date = date_format_date($dateobject, 'long');
  }
  
  // Check if we use first/last name.
  $firstname = variable_get('sign_for_acknowledgement_firstname', '');
  $firstname_table = _sign_for_acknowledgement_table_name(strtolower($firstname));
  $lastname = variable_get('sign_for_acknowledgement_lastname', '');
  $lastname_table = _sign_for_acknowledgement_table_name(strtolower($lastname));
  $use_names = _sign_for_acknowledgement_use_full_name($firstname_table, $lastname_table);
  
  $myname = $use_names ? t('Last name, first name') : t('Account');

  $header_cells = array();
  // Now we create another array where data of first cell is situated..
  $header_cells['username'] = array('data' => t('Username'), 'field' => 'name');
  // Realname cell...
  if ($use_names) {
    $header_cells['realname'] = array('data' => t('Last name, first name'), 'field' => 'lname');
  }
  // Role cell...
  $header_cells['roles'] = t('Roles');
  // Having read cell...
  $header_cells['read'] = array('data' => t('Signed'), 'field' => 'f.hid');
  // Datetime cell...
  $header_cells['date'] = array('data' => t('Date'), 'field' => 'f.mydate');
  // Status cell...
  if ($date) {
    $header_cells['status'] = t('Status');
  }
    
  // Building query with abstraction layer.
  $query = db_select('users', 'u')
    ->extend('TableSort')
    ->extend('PagerDefault')
    ->orderByHeader($header_cells)
    ->condition('u.uid', 0, '<>')
    ->condition('u.status', 1)
    ->fields('u', array('uid', 'name'))
    ->fields('f', array('hid', 'mydate'));
  if ($rows_limit != -1) {
    $query->limit($rows_limit);
  }
  if ($use_names) {
    // Add  other fields with alias...
    $query->addField('n', 'field_' . $firstname . '_value', 'fname');
    $query->addField('c', 'field_' . $lastname . '_value', 'lname');
  }

  if ($use_names) {
    $query->leftjoin($firstname_table, 'n', 'u.uid = n.entity_id');
    $query->leftjoin($lastname_table, 'c', 'u.uid = c.entity_id');
  }
  $query->leftjoin('sign_for_acknowledgement',
    'f',
    'f.user_id = u.uid AND f.node_id = :nid',
    array(':nid' => $nodeid)
  );

  $records = $query->execute()->fetchAll();

  // init array
  $rows = array();
  $roles = array_map('check_plain', user_roles(TRUE));

  foreach ($records as $record) {
    // Set up user roles.
    $users_roles = array();
    $roles_result = db_query('SELECT rid FROM {users_roles} WHERE uid = :uid', array(':uid' => $record->uid));
    foreach ($roles_result as $user_role) {
      $users_roles[] = $roles[$user_role->rid];
    }
    $rows[$record->uid] = array();
    $rows[$record->uid]['username'] = $record->name;    
    if ($use_names) {
      $rows[$record->uid]['realname'] = filter_xss($record->lname . ' ' . $record->fname);
    }
    $rows[$record->uid]['roles'] = implode(', ', $users_roles);
    $rows[$record->uid]['read'] = $record->hid ? t('YES') : t('NO');
    $rows[$record->uid]['date'] = $record->hid && $record->mydate ? format_date($record->mydate, 'short') : t('---');
    if ($date) {
      $rows[$record->uid]['status'] = _sign_for_acknowledgement_status($timestamp, $record->mydate);
    }
  }

  $form = array();
  $form['date'] = array(
    '#type' => 'markup',
    '#markup' => $date? t('To be signed within: ') . $date : '',
  );
  $form['myselector'] = array (
    '#type' => 'tableselect',
    '#header' => $header_cells,
    '#options' => $rows,
    '#attributes' => array(),
    '#empty' => t('No content available.'),
  );
  $form['node'] = array(
    '#type' => 'value',
    '#name' => 'nodeid',
    '#value' => $nodeid,
  );
  $form['unsign_submit'] = array(
    '#type' => 'submit',
    '#value' => t('remove acknowledgement'),
    '#submit' => array('sign_for_acknowledgement_table_form_submit'),
    '#prefix' => '<div style="float:left">' . t('...if selected ') . '&nbsp;</div>',
  );
  $form['sign_submit'] = array(
    '#type' => 'submit',
    '#value' => t('add acknowledgement'),
    '#submit' => array('sign_for_acknowledgement_table_sign_form_submit'),
  );
  $form['pager'] = array('#markup' => theme('pager'));
  return ($form);
}
/**
 * Settings form: presents the sign_for_acknowledgement settings page.
 */
function sign_for_acknowledgement_admin_settings_form() {
  $my_nodes = array();
  node_types_rebuild();
  $node_types = node_type_get_types();
  foreach ($node_types as $node_type) {
    $my_nodes[$node_type->type] = $node_type->type;
  }

  $form['sign_for_acknowledgement_content1'] = array(
    '#type' => 'fieldset',
    '#title' => t('Node types'),
  );
  $form['sign_for_acknowledgement_content1']['sign_for_acknowledgement_node_types'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Node types to be handled by the module'),
    '#default_value' => variable_get('sign_for_acknowledgement_node_types', array('article' => 'article')),
    '#options' => $my_nodes,
    '#multiple' => TRUE,
  );
  $form['sign_for_acknowledgement_content2'] = array(
    '#type' => 'fieldset',
    '#title' => t('Messages'),
  );
  $form['sign_for_acknowledgement_content2']['sign_for_acknowledgement_warning'] = array(
    '#type' => 'textfield',
    '#title' => t('"Remember to sign" warning'),
    '#default_value' => variable_get('sign_for_acknowledgement_warning', t('<strong>Please, remember to sign for acknowledgement.</strong>')),
  );
  $form['sign_for_acknowledgement_content2']['sign_for_acknowledgement_data_sent'] = array(
    '#type' => 'textfield',
    '#title' => t('"Data sent" message'),
    '#default_value' => variable_get('sign_for_acknowledgement_data_sent', t('Document has been signed.')),
  );
  $form['sign_for_acknowledgement_content2']['sign_for_acknowledgement_do_sign'] = array(
    '#type' => 'textfield',
    '#title' => t('"Sign for acknowledgement" message'),
    '#default_value' => variable_get('sign_for_acknowledgement_do_sign', t('Click here to confirm you have read the document')),
  );
  $form['sign_for_acknowledgement_content2']['sign_for_acknowledgement_is_signed'] = array(
    '#type' => 'textfield',
    '#title' => t('"Document already signed" message'),
    '#default_value' => variable_get('sign_for_acknowledgement_is_signed', t('You have already signed this document')),
  );
  $form['sign_for_acknowledgement_content2']['sign_for_acknowledgement_is_expired'] = array(
    '#type' => 'textfield',
    '#title' => t('"Time is expired" message'),
    '#default_value' => variable_get('sign_for_acknowledgement_is_expired', t('Can\'t sign for acknowledgement, deadline closed.')),
  );
  $form['sign_for_acknowledgement_content2']['sign_for_acknowledgement_marked'] = array(
    '#type' => 'textfield',
    '#title' => t('"Documents to be signed" message'),
    '#default_value' => variable_get('sign_for_acknowledgement_marked', t('The documents marked with *** are to be signed by you.')),
  );
  $form['sign_for_acknowledgement_content3'] = array(
    '#type' => 'fieldset',
    '#title' => t('User'),
  );
  $infos = field_info_instances('user', 'user');
  $my_fields = array();
  $my_fields[''] = t('No field');
  foreach ($infos as $info) {
    $my_fields[$info['label']] = $info['label'];
  }
  $form['sign_for_acknowledgement_content3']['sign_for_acknowledgement_firstname'] = array(
    '#type' => 'select',
    '#title' => t('Select first name field'),
    '#default_value' => variable_get('sign_for_acknowledgement_firstname', ''),
    '#options' => $my_fields,
  );
  $form['sign_for_acknowledgement_content3']['sign_for_acknowledgement_lastname'] = array(
    '#type' => 'select',
    '#title' => t('Select last name field'),
    '#default_value' => variable_get('sign_for_acknowledgement_lastname', ''),
    '#options' => $my_fields,
  );
  $form['sign_for_acknowledgement_content4'] = array(
    '#type' => 'fieldset',
    '#title' => t('Layout'),
  );
  $form['sign_for_acknowledgement_content4']['sign_for_acknowledgement_limit'] = array(
    '#type' => 'select',
    '#title' => t('Number of rows in the table'),
    '#default_value' => variable_get('sign_for_acknowledgement_limit', 10),
    '#options' => array(
      5 => 5,
      10 => 10,
      50 => 50,
      100 => 100,
      500 => 500,
      1000 => 1000,
      -1 => t('All'),
    ),
  );
  $form['sign_for_acknowledgement_content4']['sign_for_acknowledgement_weight'] = array(
    '#type' => 'textfield',
    '#title' => t('Checkbox and text weight'),
    '#size' => 4,
    '#element_validate' => array('element_validate_number'),
    '#default_value' => variable_get('sign_for_acknowledgement_weight', 1),
  );
  $form['sign_for_acknowledgement_content4']['sign_for_acknowledgement_show_submit'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show the submit button?'),
    '#defaul_value' => variable_get('sign_for_acknowledgement_show_submit', FALSE),
    '#attributes' => variable_get('sign_for_acknowledgement_show_submit', FALSE) ? array('checked' => 'checked') : NULL,
  );
  $form['sign_for_acknowledgement_content4']['sign_for_acknowledgement_block_expired'] = array(
    '#type' => 'checkbox',
    '#title' => t('Prevent the signature if terms have expired.'),
    '#defaul_value' => variable_get('sign_for_acknowledgement_block_expired', FALSE),
    '#attributes' => variable_get('sign_for_acknowledgement_block_expired', FALSE) ? array('checked' => 'checked') : NULL,
  );
  $form['#submit'][] = '_sign_for_acknowledgement_reset_expirations';
  return system_settings_form($form);
}
